import UIKit
import RxSwift

let disposeBag = DisposeBag()

extension String {
    func pad(_ length: Int) -> String {
        return self.padding(toLength: length,
                            withPad: " ",
                            startingAt: 0)
    }
}

func printOriginalQueue(from elements: [Int]) {
    var originalQueue = "--"
    
    for i in elements {
        originalQueue += " \(i) --"
    }
    
    print ("\("Original".pad(15))| \(originalQueue)>")
}

func example<T>(of name: String, _ observable: T) where T: ObservableType, T.Element == Int {
    var subscriptionResult = "--"
    
    observable
        .subscribe(
            onNext: { value in
                subscriptionResult += " \(value) --"
        },
            onCompleted: {
                print("\(name.pad(15))| \(subscriptionResult)>")
        })
        .disposed(by: disposeBag)
}

print("# Filtering operators\n")

let elements = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
printOriginalQueue(from: elements)

let observable = Observable.from(elements)

example(of: "elementAt",
        observable.elementAt(1)
)

example(of: "filter",
        observable.filter { $0 % 2 == 0 }
)

example(of: "skip",
        observable.skip(3)
)

example(of: "take",
        observable.take(2)
)

let elementsToDistinct = [0, 1, 1, 1, 2, 2, 1, 1,]
let distinctObservable = Observable.from(elementsToDistinct)

print("")
printOriginalQueue(from: elementsToDistinct)

example(of: "distinctUntilChanged",
        distinctObservable.distinctUntilChanged()
)

print("\n# Transformation operators\n")

printOriginalQueue(from: elements)

example(of: "map",
        observable.map { $0 * 10 }
)

example(of: "reduce",
        observable.reduce(0, accumulator: +)
)

print("\n# Combination operators\n")

let elements1 = [0, 1, 2, 3, 4]
let elements2 = [5, 6, 7, 8, 9]

printOriginalQueue(from: elements1)
printOriginalQueue(from: elements2)

let observable1 = Observable.from(elements1)
let observable2 = Observable.from(elements2)

example(of: "merge",
        Observable.merge(observable1, observable2)
)

example(of: "concat",
        Observable.concat(observable1, observable2)
)

example(of: "combineLatest",
        Observable.combineLatest(observable1, observable2, resultSelector: { $0 * $1 })
)
